; *** RAM reserve ***
License_addr1_LSB  = $E0
License_addr1_MSB  = $E1
License_addr2_LSB  = $E2
License_addr2_MSB  = $E3

;******************************************************************************
; main
;******************************************************************************
License_main:
  inc <currentActivity
  ldx <currentActivity
  lda activities, x
  asl a
  tax
  lda License_main_VECTOR, x
  sta <address1_LSB
  inx
  lda License_main_VECTOR, x
  sta <address1_MSB
  jmp [address1_LSB]

License_main_VECTOR:
  .dw License_init                ; #0 
  .dw License_wait0               ; #1 wait
  .dw License_fadein              ; #2 tF[hC
  .dw License_getMessageAddress   ; #3 bZ[Wf[^̃AhX擾
  .dw License_displayMessage      ; #4 bZ[Wópbh͑҂
  .dw License_wait1               ; #5 waitpbh͑҂
  .dw License_fadeout             ; #6 tF[hAEg
  .dw License_wait0               ; #7 wait
  .dw License_terminate           ; #8 terminate(next activity)


;******************************************************************************
; initialize (activity #0)
;******************************************************************************
License_init:
  lda #0
  sta Snd_bgm
  sta Snd_sfx
  jsr MMC_changeCHR_High

  ; clear palette
  lda REG_PPU_2
  lda #$3F
  sta REG_PPU_6
  lda #$00
  sta REG_PPU_6
  ldy #$20
  lda #$0F
.loop0:
  sta REG_PPU_7
  dey
  bne .loop0

  ; load logo
  lda REG_PPU_2
  lda #$23
  sta REG_PPU_6
  lda #$C0
  sta REG_PPU_6
  ldx #$00
  lda #$FF
.loop1:
  sta REG_PPU_7
  inx
  cpx #$20
  bcc .loop1
  
  lda #$A9
  sta <address1_LSB
  lda #$20
  sta <address1_MSB
  lda #$00
  tay
.loop2:
  lda REG_PPU_2
  lda <address1_MSB
  sta REG_PPU_6
  lda <address1_LSB
  sta REG_PPU_6
.loop3:
  lda License_logo, y
  sta REG_PPU_7
  iny
  cpy #$80
  bcc .continue
  ldx <currentActivity
  lda #0
  sta processCounters, x
  inc activities, x
  rts
.continue:
  tya
  and #$0F
  bne .loop3
  lda <address1_LSB
  clc
  adc #$20
  sta <address1_LSB
  bcc .loop2
  inc <address1_MSB
  jmp .loop2


;******************************************************************************
; wait #0 (activity #1, #7)
;******************************************************************************
License_wait0:
  ldx <currentActivity
  lda processCounters, x
  cmp #16 ; wait frames
  bcc .continue
  lda #0
  sta processCounters, x
  inc activities, x
  rts
.continue:
  inc processCounters, x
  rts


;******************************************************************************
; fade in (activity #2)
;******************************************************************************
License_fadein:
  lda #low(License_pallet)
  sta <address1_LSB
  lda #high(License_pallet)
  sta <address1_MSB
  jmp Util_fadein


;******************************************************************************
; get Message Address (activity #3)
;******************************************************************************
License_getMessageAddress:
  lda #low(License_message)
  sta <License_addr1_LSB
  lda #high(License_message)
  sta <License_addr1_MSB
  lda #$00
  sta <License_addr2_LSB
  lda #$22
  sta <License_addr2_MSB

  lda #1
  sta Snd_bgm
  ldx <currentActivity
  inc activities, x
  rts


;******************************************************************************
; display message (activity #4)
;******************************************************************************
License_displayMessage:
  ldx <currentActivity
  lda <pad1trig
  and #$90 ; pad1 A{^ or Start
  beq .continue
  lda #6
  sta activities, x
.continue:
  jmp License_drawChar


;******************************************************************************
; wait #1 (activity #5)
;******************************************************************************
License_wait1:
  ldx <currentActivity
  lda processCounters, x
  cmp #128
  bcc .padScan
  inc activities, x
  lda #0
  sta processCounters, x
  rts
.padScan:
  lda <pad1trig
  and #$90 ; pad1 A{^ or Start
  beq .continue
  inc activities, x
  lda #0
  sta processCounters, x
  rts
.continue:
  inc processCounters, x
  rts


;******************************************************************************
; fade out (activity #6)
;******************************************************************************
License_fadeout:
  lda #low(License_pallet)
  sta <address1_LSB
  lda #high(License_pallet)
  sta <address1_MSB
  jmp Util_fadeout


;******************************************************************************
; terminate (activity #8)
;******************************************************************************
License_terminate:
  lda #0
  sta Snd_bgm
  ldx <currentActivity
  sta <License_addr1_LSB
  sta <License_addr1_MSB
  sta <License_addr2_LSB
  sta <License_addr2_MSB
  sta activities, x
  sta processCounters, x
  dex
  inc activities, x
  sta processCounters, x
  rts


;******************************************************************************
; draw char
;******************************************************************************
License_drawChar:
  lda REG_PPU_2
  lda <License_addr2_MSB
  sta REG_PPU_6
  lda <License_addr2_LSB
  sta REG_PPU_6

  ldy #0
.read:
  lda [License_addr1_LSB], y
  cmp #$F0
  beq .cmdF0
  cmp #$FF
  beq .cmdFF
  sta REG_PPU_7
.inc1:
  inc <License_addr1_LSB
  bne .inc2
  inc <License_addr1_MSB
.inc2:
  inc <License_addr2_LSB
  bne .continue
  inc <License_addr2_MSB
.continue:
  rts
.cmdF0:
  lda <License_addr2_LSB
  clc
  adc #$20
  and #$E0
  sta <License_addr2_LSB
.inc3:
  inc <License_addr1_LSB
  bne .continue
  inc <License_addr2_LSB
  bne .continue
.cmdFF:
  ldx <currentActivity
  lda #0
  sta processCounters, x
  inc activities, x
  rts


;******************************************************************************
; data
;******************************************************************************
License_pallet:
  .db $0F, $0F, $15, $30, $0F, $0F, $0F, $0F, $0F, $0F, $0F, $0F, $0F, $05, $36, $30
  .db $0F, $0F, $0F, $0F, $0F, $0F, $0F, $0F, $0F, $0F, $0F, $0F, $0F, $0F, $0F, $0F

License_logo:
  ;   +0   +1   +2   +3   +4   +5   +6   +7   +8   +9   +A   +B   +C   +D   +E   +F
  .db $00, $00, $82, $83, $84, $85, $86, $00, $00, $00, $8A, $8B, $8C, $8D, $8E, $00
  .db $90, $91, $92, $93, $94, $95, $96, $97, $98, $00, $9A, $9B, $9C, $9D, $9E, $9F
  .db $A0, $A1, $A2, $A3, $A4, $A5, $A6, $A7, $A8, $A9, $AA, $AB, $AC, $AD, $AE, $AF
  .db $B0, $B1, $B2, $B3, $B4, $B5, $B6, $B7, $B8, $00, $BA, $BB, $BC, $BD, $BE, $BF
  .db $C0, $C1, $C2, $C3, $C4, $C5, $C6, $C7, $C8, $C9, $CA, $CB, $CC, $CD, $CE, $CF
  .db $00, $00, $00, $00, $00, $00, $00, $00, $D8, $D9, $DA, $DB, $DC, $DD, $DE, $DF
  .db $00, $00, $00, $00, $00, $00, $00, $00, $00, $E9, $EA, $EB, $EC, $ED, $EE, $EF
  .db $00, $00, $00, $00, $00, $00, $00, $00, $00, $F9, $FA, $FB, $FC, $FD, $FE, $FF

License_message:
  ;   +0   +1   +2   +3   +4   +5   +6   +7   +8   +9   +A   +B   +C   +D   +E   +F
  .db $00, $1D, $23, $1D, $1E, $0F, $17, $02, $27, $02, $00, $29, $2A, $2B, $2C, $2D
  .db $2E, $2F, $F0
  .db $F0
  .db $00, $00, $00, $00, $00, $00, $00, $00, $38, $00, $30, $31, $32, $33, $34, $35
  .db $33, $00, $34, $36, $37, $31, $32, $33, $00, $38, $00, $00, $00, $00, $00, $00
  .db $F0
  .db $00, $1E, $12, $13, $1D, $00, $1A, $1C, $19, $11, $1C, $0B, $17, $00, $13, $1D
  .db $00, $10, $1C, $0F, $0F, $00, $1D, $19, $10, $1E, $21, $0B, $1C, $0F, $25, $00
  .db $00, $23, $19, $1F, $00, $0D, $0B, $18, $00, $1C, $0F, $0E, $13, $1D, $1E, $1C
  .db $13, $0C, $1F, $1E, $0F, $00, $13, $1E, $00, $0B, $18, $0E, $26, $19, $1C, $00
  .db $00, $17, $19, $0E, $13, $10, $23, $00, $13, $1E, $00, $1F, $18, $0E, $0F, $1C
  .db $00, $1E, $12, $0F, $00, $1E, $0F, $1C, $17, $1D, $00, $19, $10, $00, $00, $00
  .db $00, $1E, $12, $0F, $00, $39, $3A, $30, $00, $3B, $3C, $00, $19, $1C, $00, $16
  .db $0B, $1E, $0F, $1C, $27, $FF

